'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.getOneModel = exports.validateModelNames = exports.createModel = exports.getDB = exports.FIXTURE_FILE_NAME = exports.MODEL_NAME_FIELD = exports.MODEL_TABLE_NAME = undefined;

var _lokijs = require('lokijs');

var _lokijs2 = _interopRequireDefault(_lokijs);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _collection = require('./collection');

var collectionUtils = _interopRequireWildcard(_collection);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var MODEL_TABLE_NAME = exports.MODEL_TABLE_NAME = 'models';
var MODEL_NAME_FIELD = exports.MODEL_NAME_FIELD = 'modelName';
var FIXTURE_FILE_NAME = exports.FIXTURE_FILE_NAME = 'db.json';

var db = void 0;

/**
 * Initalizes the LokiJS database instance
 * @returns {Promise.<*>}
 */
var getDB = exports.getDB = function getDB() {
    if (!db) {
        db = new _lokijs2.default(process.cwd() + '/src/data/' + FIXTURE_FILE_NAME, {
            autosave: true,
            autosaveInterval: 100
        });
        db.save();
    }
    return db;
};

/**
 * Creates a model in database model collection if it does not exist
 * Also creates the model table if that table does not exist
 * @param db database instance
 * @param definition parsed swagger model definition
 * @returns {object} inserted model instance
 */
var createModel = exports.createModel = function createModel(db, definition) {
    var modelCollection = void 0;

    try {
        modelCollection = collectionUtils.findCollection(db, MODEL_TABLE_NAME);
    } catch (e) {
        if (e.message === 'MODEL_DOES_NOT_EXIST') {
            modelCollection = collectionUtils.createCollection(db, MODEL_TABLE_NAME, { unique: [MODEL_NAME_FIELD] });
        } else {
            console.error('Something went wrong while creating the models table.');
        }
    }

    var foundModel = collectionUtils.findOne(modelCollection, MODEL_NAME_FIELD, definition.modelName);

    if (!foundModel) {
        modelCollection.insert(definition);
        foundModel = collectionUtils.findOne(modelCollection, MODEL_NAME_FIELD, definition.modelName);
    }
    return foundModel;
};

/**
 * Ensure there are no duplicates in model names
 * @param models array of model names
 * @returns {Array}
 */
var validateModelNames = exports.validateModelNames = function validateModelNames(models) {
    if (_lodash2.default.uniq(models).length !== _lodash2.default.keys(models).length) {
        console.warn('BEWARE, MODELS ARE DUPLICATED BETWEEN SWAGGERS. REMOVING DUPLICATES.');
    }
    return _lodash2.default.uniq(models);
};

/**
 * Get one model from DB
 * @param db
 * @param modelName
 */
var getOneModel = exports.getOneModel = function getOneModel(db, modelName) {
    var modelTable = collectionUtils.findCollection(db, MODEL_TABLE_NAME);
    return collectionUtils.findOne(modelTable, MODEL_NAME_FIELD, modelName);
};