'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _koaRouter = require('koa-router');

var _koaRouter2 = _interopRequireDefault(_koaRouter);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _collection = require('../db/collection');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Put all the other stuff down there together
 *
 * @param db
 * @param basePath
 * @param paths
 * @returns {*}
 */
exports.default = function (db, _ref) {
    var basePath = _ref.basePath,
        paths = _ref.paths;

    //remove last slash of route if there is one
    var cleanedBasePath = basePath[basePath.length - 1] !== '/' ? basePath : basePath.slice(0, -1);

    //create router container
    var router = (0, _koaRouter2.default)({
        prefix: cleanedBasePath || ''
    });

    _lodash2.default.each(paths, function (pathOptions, route) {
        //get methods without parameters
        var routerMethods = _lodash2.default.compact(_lodash2.default.map(_lodash2.default.keys(pathOptions), function (key) {
            return key !== 'parameters' ? key : null;
        }));

        // mount methods to router if it has any
        if (routerMethods.length) {
            router = addMethodsToRouter(db, router, { route: route, routerMethods: routerMethods, pathOptions: pathOptions });
        }
    });

    return router;
};

/**
 * Add methods + bind callbacks to the router + bind parameters
 * @param db
 * @param router
 * @param route
 * @param routerMethods
 * @param pathOptions
 * @returns {*}
 */


var addMethodsToRouter = function addMethodsToRouter(db, router, _ref2) {
    var route = _ref2.route,
        routerMethods = _ref2.routerMethods,
        pathOptions = _ref2.pathOptions;

    var formattedRouteParameters = createAliasesForParameters(route);
    var formattedRoute = _lodash2.default.reduce(formattedRouteParameters, function (acc, elem) {
        return acc.replace(elem.name, ':' + elem.alias).replace(/[{}]/g, '');
    }, route);

    var routerRef = router;

    _lodash2.default.each(routerMethods, function (method) {
        //create route
        var _pathOptions$method = pathOptions[method],
            responses = _pathOptions$method.responses,
            parameters = _pathOptions$method.parameters;

        var getter = responseGetter(db, responses);

        router[method](formattedRoute, getter);

        //add parameters link to route if there are any to add
        if (formattedRouteParameters) {
            var swaggerOptions = _lodash2.default.filter(pathOptions[method].parameters, { in: 'path' });
            router = addUrlParamToRouter(router, formattedRouteParameters, swaggerOptions);
        }

        //add handlers to route
    });

    return routerRef;
};

/**
 * Create route callback function, defining randomly how many fixtures we will return
 * and which one exactly from the local db
 *
 * @param db
 * @param responses
 * @returns {function(*)}
 */
var responseGetter = function responseGetter(db, responses) {
    var body = void 0;
    var defaultSuccess = responses[200] || responses[201] || responses[204];

    return function (ctx) {
        if (defaultSuccess && defaultSuccess.schema) {
            var objectRef = getReferencesAndCount(defaultSuccess.schema);
            var collection = (0, _collection.findCollection)(db, objectRef.object);

            body = _lodash2.default.take(_lodash2.default.shuffle(_lodash2.default.map(collection.find(), '' + objectRef.object)), objectRef.fixtureCount);
            if (defaultSuccess.schema.type === 'array') {
                ctx.body = body;
            } else if (defaultSuccess.schema['$ref']) {
                ctx.body = body[0];
            }
            return;
        } else if (defaultSuccess) {
            var possibleCodes = [200, 201, 204];
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                for (var _iterator = (0, _getIterator3.default)(possibleCodes), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    var code = _step.value;

                    if (responses[code]) {
                        ctx.status = code;
                        break;
                    }
                }
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator.return) {
                        _iterator.return();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }

            return;
        }
        ctx.throw('Unsupported format! Can you create an issue in Github about that?');
    };

    //2 handle parameters in body (in: body)
    //  -> is body required ?
    //  -> validate against schema
    //3 define if should give success or error through override
    //  -> if so, do return error from schema
    //4 get smallest response code and accept it as default one
};

/**
 * Get the fixture that should be return + a random number
 *
 * @param reference
 * @returns {{fixtureCount: number, object}}
 */
var getReferencesAndCount = function getReferencesAndCount(reference) {
    if (reference['$ref']) {
        var object = reference['$ref'].replace('#/definitions/', '');
        return { fixtureCount: 1, object: object };
    } else if (reference.items) {
        var _object = reference.items['$ref'].replace('#/definitions/', '');
        return { fixtureCount: Math.round(Math.random() * 5) || 1, object: _object };
    } else {
        return { fixtureCount: 0, object: reference.description };
    }
};

/**
 * Format parameters
 *
 * @param route
 * @returns {*}
 */
var createAliasesForParameters = function createAliasesForParameters(route) {
    var urlParams = route.match(/{(.*)}/g);

    return _lodash2.default.reduce(urlParams, function (acc, param) {
        var rawParameter = param.replace(/[{}]/g, '');

        if (!acc[rawParameter]) {
            acc[rawParameter] = {
                name: rawParameter,
                alias: param.replace(/\-/g, '')
            };
        }
        return acc;
    }, {});
};

/**
 * Binds parameters to the router so they can be correctly recognized on query
 * @param router
 * @param formattedRouteParameters
 * @param swaggerOptions
 * @returns {*}
 */
var addUrlParamToRouter = function addUrlParamToRouter(router, formattedRouteParameters, swaggerOptions) {
    var routerRef = router;

    _lodash2.default.each(formattedRouteParameters, function (_ref3) {
        var name = _ref3.name;

        //get details of parameter (required, type...)
        var swaggerOptionsForParameter = _lodash2.default.find(swaggerOptions, { name: name });

        routerRef.param(name, function () {
            var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(value, ctx, next) {
                return _regenerator2.default.wrap(function _callee$(_context) {
                    while (1) {
                        switch (_context.prev = _context.next) {
                            case 0:
                                if (!swaggerOptionsForParameter.required) {
                                    _context.next = 3;
                                    break;
                                }

                                if (value) {
                                    _context.next = 3;
                                    break;
                                }

                                return _context.abrupt('return', ctx.throw('Parameter ' + name + ' is missing'));

                            case 3:
                                return _context.abrupt('return', next());

                            case 4:
                            case 'end':
                                return _context.stop();
                        }
                    }
                }, _callee, undefined);
            }));

            return function (_x, _x2, _x3) {
                return _ref4.apply(this, arguments);
            };
        }());
    });

    return routerRef;
};