'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _swaggerParser = require('swagger-parser');

var _swaggerParser2 = _interopRequireDefault(_swaggerParser);

var _yamljs = require('yamljs');

var _yamljs2 = _interopRequireDefault(_yamljs);

var _db = require('../db');

var _fixtures = require('../db/fixtures');

var _createRouterFromSwagger = require('./createRouterFromSwagger');

var _createRouterFromSwagger2 = _interopRequireDefault(_createRouterFromSwagger);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var SOURCE_FILES_LOCATION = process.cwd() + '/src/data';

/**
 * Read swagger files in data directory and pass them to the fixture + routes creator
 * @returns {Promise.<void>}
 */
exports.default = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2() {
    var localSources, swaggers;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
        while (1) {
            switch (_context2.prev = _context2.next) {
                case 0:
                    _context2.prev = 0;
                    _context2.next = 3;
                    return (0, _utils.readDir)(SOURCE_FILES_LOCATION, function (files) {
                        return _lodash2.default.filter(files, function (file) {
                            return !file.includes('morphling-override') && !['db.json', '.gitkeep'].includes(file);
                        });
                    });

                case 3:
                    localSources = _context2.sent;
                    _context2.next = 6;
                    return _promise2.default.all(_lodash2.default.map(localSources, function () {
                        var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(fileName) {
                            var fileContent;
                            return _regenerator2.default.wrap(function _callee$(_context) {
                                while (1) {
                                    switch (_context.prev = _context.next) {
                                        case 0:
                                            _context.next = 2;
                                            return (0, _utils.readFile)(SOURCE_FILES_LOCATION + '/' + fileName);

                                        case 2:
                                            fileContent = _context.sent;

                                            if (!fileName.includes('.json')) {
                                                _context.next = 7;
                                                break;
                                            }

                                            return _context.abrupt('return', JSON.parse(fileContent));

                                        case 7:
                                            if (!(fileName.includes('.yml') || fileName.includes('.yaml'))) {
                                                _context.next = 9;
                                                break;
                                            }

                                            return _context.abrupt('return', _yamljs2.default.parse(fileContent));

                                        case 9:
                                        case 'end':
                                            return _context.stop();
                                    }
                                }
                            }, _callee, undefined);
                        }));

                        return function (_x) {
                            return _ref2.apply(this, arguments);
                        };
                    }()));

                case 6:
                    swaggers = _context2.sent;
                    _context2.next = 9;
                    return Morphling(swaggers);

                case 9:
                    return _context2.abrupt('return', _context2.sent);

                case 12:
                    _context2.prev = 12;
                    _context2.t0 = _context2['catch'](0);

                    console.log('Error while fetching local swaggers', _context2.t0);

                case 15:
                case 'end':
                    return _context2.stop();
            }
        }
    }, _callee2, undefined, [[0, 12]]);
}));

/**
 * Takes an array of swaggers in, returns a router with bound fixtures out
 * @param swaggers
 * @returns {Promise.<void>}
 * @constructor
 */

var Morphling = function () {
    var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(swaggers) {
        var db, parsedSwaggers, allModelDefinitions, cleanedModelDefinitions, fixtures;
        return _regenerator2.default.wrap(function _callee3$(_context3) {
            while (1) {
                switch (_context3.prev = _context3.next) {
                    case 0:
                        db = (0, _db.getDB)();
                        _context3.prev = 1;
                        _context3.next = 4;
                        return _promise2.default.all(_lodash2.default.map(swaggers, function (elem) {
                            return parseSwagger(elem);
                        }));

                    case 4:
                        parsedSwaggers = _context3.sent;
                        allModelDefinitions = _lodash2.default.merge.apply(_lodash2.default, (0, _toConsumableArray3.default)(_lodash2.default.map(parsedSwaggers, function (swagger) {
                            return _lodash2.default.map(swagger.definitions, function (def, defName) {
                                return (0, _extends3.default)({ modelName: defName }, def);
                            });
                        })));
                        cleanedModelDefinitions = (0, _db.validateModelNames)(allModelDefinitions);
                        fixtures = (0, _fixtures.createFixturesFromDefinitions)(db, cleanedModelDefinitions);
                        return _context3.abrupt('return', _lodash2.default.map(swaggers, function (source) {
                            var basePath = source.basePath,
                                paths = source.paths;

                            return (0, _createRouterFromSwagger2.default)(db, { basePath: basePath, paths: paths });
                        }));

                    case 11:
                        _context3.prev = 11;
                        _context3.t0 = _context3['catch'](1);

                        console.error(_context3.t0);

                    case 14:
                    case 'end':
                        return _context3.stop();
                }
            }
        }, _callee3, undefined, [[1, 11]]);
    }));

    return function Morphling(_x2) {
        return _ref3.apply(this, arguments);
    };
}();

/**
 * Swagger parser, dont not parse circular references because JSON cannot handle that
 * @param source
 * @returns {Promise.<*>}
 */
var parseSwagger = function () {
    var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(source) {
        return _regenerator2.default.wrap(function _callee4$(_context4) {
            while (1) {
                switch (_context4.prev = _context4.next) {
                    case 0:
                        _context4.next = 2;
                        return _swaggerParser2.default.bundle(source, { $refs: { circular: 'ignore' } });

                    case 2:
                        return _context4.abrupt('return', _context4.sent);

                    case 3:
                    case 'end':
                        return _context4.stop();
                }
            }
        }, _callee4, undefined);
    }));

    return function parseSwagger(_x3) {
        return _ref4.apply(this, arguments);
    };
}();