'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.readDir = exports.checkFileExistence = exports.writeFile = exports.writeFileStream = exports.readFile = undefined;

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Read file content in path
 * @param path
 * @returns {Promise}
 */
var readFile = exports.readFile = function () {
    var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(path) {
        return _regenerator2.default.wrap(function _callee$(_context) {
            while (1) {
                switch (_context.prev = _context.next) {
                    case 0:
                        return _context.abrupt('return', new _promise2.default(function (resolve, reject) {
                            _fs2.default.readFile(path, 'utf8', function (err, data) {
                                if (err) {
                                    return reject(err);
                                }
                                return resolve(data);
                            });
                        }));

                    case 1:
                    case 'end':
                        return _context.stop();
                }
            }
        }, _callee, undefined);
    }));

    return function readFile(_x) {
        return _ref.apply(this, arguments);
    };
}();
/**
 * Write file in path from other file
 * @returns {Promise}
 * @param fileName
 * @param file
 */
var writeFileStream = exports.writeFileStream = function () {
    var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(fileName, file) {
        var readStream, writeStream;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
            while (1) {
                switch (_context2.prev = _context2.next) {
                    case 0:
                        _context2.prev = 0;
                        readStream = _fs2.default.createReadStream(file.path);
                        writeStream = _fs2.default.createWriteStream(process.cwd() + '/src/data/' + fileName);

                        readStream.pipe(writeStream);
                        _fs2.default.unlink(file.path);

                        _context2.next = 7;
                        return readFile(process.cwd() + '/src/data/' + fileName);

                    case 7:
                        return _context2.abrupt('return', _context2.sent);

                    case 10:
                        _context2.prev = 10;
                        _context2.t0 = _context2['catch'](0);

                        console.log('Error while writing file!', _context2.t0);

                    case 13:
                    case 'end':
                        return _context2.stop();
                }
            }
        }, _callee2, undefined, [[0, 10]]);
    }));

    return function writeFileStream(_x2, _x3) {
        return _ref2.apply(this, arguments);
    };
}();

/**
 * Write file in path
 * @param filename file name
 * @param content JSON object
 * @returns {Promise}
 */
var writeFile = exports.writeFile = function () {
    var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(filename, content) {
        var contentToWrite;
        return _regenerator2.default.wrap(function _callee3$(_context3) {
            while (1) {
                switch (_context3.prev = _context3.next) {
                    case 0:
                        contentToWrite = _lodash2.default.isObject(content) ? (0, _stringify2.default)(content) : content;
                        return _context3.abrupt('return', new _promise2.default(function (resolve, reject) {
                            _fs2.default.writeFile(process.cwd() + '/src/data/' + filename, contentToWrite, 'utf8', function (err, res) {
                                if (err) {
                                    return reject(err);
                                }
                                return resolve(res);
                            });
                        }));

                    case 2:
                    case 'end':
                        return _context3.stop();
                }
            }
        }, _callee3, undefined);
    }));

    return function writeFile(_x4, _x5) {
        return _ref3.apply(this, arguments);
    };
}();

/**
 * check if file exists
 * @returns {Promise}
 * @param path
 */
var checkFileExistence = exports.checkFileExistence = function () {
    var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(path) {
        return _regenerator2.default.wrap(function _callee4$(_context4) {
            while (1) {
                switch (_context4.prev = _context4.next) {
                    case 0:
                        return _context4.abrupt('return', new _promise2.default(function (resolve) {
                            _fs2.default.stat(path, function (e) {
                                if (e) return resolve(false);
                                return resolve(true);
                            });
                        }));

                    case 1:
                    case 'end':
                        return _context4.stop();
                }
            }
        }, _callee4, undefined);
    }));

    return function checkFileExistence(_x6) {
        return _ref4.apply(this, arguments);
    };
}();

/**
 * Read directory file names
 * @param path path to directory
 * @param exclude files to filter out of exit
 * @returns {Promise} resolved filenames
 */
var readDir = exports.readDir = function () {
    var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(path, exclude) {
        return _regenerator2.default.wrap(function _callee5$(_context5) {
            while (1) {
                switch (_context5.prev = _context5.next) {
                    case 0:
                        return _context5.abrupt('return', new _promise2.default(function (resolve, reject) {
                            _fs2.default.readdir(path, function (err, files) {
                                if (err) {
                                    return reject(err);
                                }

                                if (exclude) {
                                    if (typeof exclude === 'function') {
                                        return resolve(exclude(files));
                                    }

                                    return resolve(_lodash2.default.filter(files, function (file) {
                                        return !exclude.includes(file);
                                    }));
                                }

                                return resolve(files);
                            });
                        }));

                    case 1:
                    case 'end':
                        return _context5.stop();
                }
            }
        }, _callee5, undefined);
    }));

    return function readDir(_x7, _x8) {
        return _ref5.apply(this, arguments);
    };
}();